/*
 * foc.h
 *
 *  Created on: May 12, 2020
 *      Author: a0232121
 */

#ifndef FOC_H_
#define FOC_H_

//#include "libraries/math/include/math.h"

#define PI 3.14159265358979
#define TWO_PI 6.28318530718

#define  ONEbySQRT3   0.57735026918963    /* 1/sqrt(3) */

#define SVGEN_SQRT3_OVER_2     0.8660254038

// Math functions

#ifdef S_FUNCTION_NAME
#define FMAX(a,b) (a>b?a:b)
#define FMIN(a,b) (a<b?a:b)
#define ATAN2F atan2f
#define SQRT sqrtf
#else
#ifndef __TMS320C28XX_CLA__
#define FMAX __fmax
#define FMIN __fmin
#define ATAN2F atan2f
#define SQRT __sqrt
#else
#define FMAX __mmaxf32
#define FMIN __mminf32
#define ATAN2F CLAatan2_inline
#define SQRT CLAsqrt_inline
#endif
#endif


//*****************************************************************************
//
//! \brief Defines the PI controller object
//
//*****************************************************************************
typedef struct _PI_Obj_
{
    float32_t Kp;                //!< the proportional gain for the PI controller
    float32_t Ki;                //!< the integral gain for the PI controller
    float32_t Ui;                //!< the integrator start value for the PI
                               //!< controller
    float32_t refValue;          //!< the reference input value
    float32_t fbackValue;        //!< the feedback input value
    float32_t ffwdValue;         //!< the feedforward input value
    float32_t outMin;            //!< the minimum output value allowed for the PI
                               //!< controller
    float32_t outMax;            //!< the maximum output value allowed for the PI
                               //!< controller
} PI_Obj;


//*****************************************************************************
//
//! \brief Defines the Motor data
//
//*****************************************************************************
typedef struct _Motor_t_
{
    float32_t Ls_d;
    float32_t Ls_q;
    float32_t Rs;
    float32_t Phi_e;

    float32_t I_abc_A[3];         //!< the current values
    float32_t V_abc_V[3];         //!< the voltage values

    float32_t I_scale;
    float32_t V_scale;

    float32_t dcBus_V;       //!< the dcBus value
    float32_t oneOverDcBus_invV;         //!< the DC Bus inverse, 1/V

    float32_t I_ab_A[2];         //!< the current values
    float32_t I_dq_A[2];         //!< the current values

    float32_t theta_e;         //!< the rotor position
    float32_t omega_e;         //!< the current values

    float32_t Sine;
    float32_t Cosine;

    float32_t Vff_dq_V[2];          //!< PI_dq FF value for decoupling
    float32_t Vout_dq_V[2];         //!< the current values
    float32_t Vout_ab_V[2];         //!< the current values

    PI_Obj pi_id;
    PI_Obj pi_iq;

    float32_t vqLimit;
    float32_t modulationLimitSquare;
} Motor_t;


//*****************************************************************************
//
//! \brief Defines the PWM data
//
//*****************************************************************************
typedef struct _PWMData_t_
{
    float32_t Vabc_pu[3];     //!< the PWM time-durations for each motor phase
    float32_t modulationLimit;
    uint16_t  cmpValue[3];
    uint16_t  deadband[3];
    uint16_t  noiseWindow;
    uint16_t  period;
    uint16_t  socCMP;
//    bool      flagEnablePwm;
} PWMData_t;

//*****************************************************************************
//
//! \brief Defines the first-order filter (FILTER_FO) object
//
//*****************************************************************************
typedef struct _FILTER_FO_Obj_
{
    float32_t a1;       //!< the denominator filter coefficient value for z^(-1)
    float32_t b0;       //!< the numerator filter coefficient value for z^0
    float32_t b1;       //!< the numerator filter coefficient value for z^(-1)
    float32_t x1;       //!< the input value at time sample n=-1
    float32_t y1;       //!< the output value at time sample n=-1
} FILTER_FO_Obj;

static inline void clarke_run(Motor_t * in)
{
    in->I_ab_A[0] = in->I_abc_A[0];
    in->I_ab_A[1] = ((in->I_abc_A[1] - in->I_abc_A[2]) * ONEbySQRT3);
}


static inline void park_run(Motor_t * in)
{
    in->I_dq_A[0] = (in->I_ab_A[0] * in->Cosine) + (in->I_ab_A[1] * in->Sine);
    in->I_dq_A[1] = (in->I_ab_A[1] * in->Cosine) - (in->I_ab_A[0] * in->Sine);
}


static inline void ipark_run(Motor_t * in)
{
    in->Vout_ab_V[0] = (in->Vout_dq_V[0] * in->Cosine) - (in->Vout_dq_V[1] * in->Sine);
    in->Vout_ab_V[1] = (in->Vout_dq_V[1] * in->Cosine) + (in->Vout_dq_V[0] * in->Sine);
}


static inline void SVGEN_run(Motor_t * m, PWMData_t * pwm)
{
    float32_t Vmax_pu = 0,Vmin_pu = 0,Vcom_pu;

    float32_t Va_pu = m->Vout_ab_V[0] * m->oneOverDcBus_invV;
    float32_t Vbeta_pu = m->Vout_ab_V[1] * m->oneOverDcBus_invV;

    float32_t Va_tmp = (float32_t)(0.5) * (-Va_pu);
    float32_t Vb_tmp = SVGEN_SQRT3_OVER_2 * Vbeta_pu;

    //
    // -0.5*Valpha + sqrt(3)/2 * Vbeta
    //
    float32_t Vb_pu = Va_tmp + Vb_tmp;

    //
    // -0.5*Valpha - sqrt(3)/2 * Vbeta
    float32_t Vc_pu = Va_tmp - Vb_tmp;

    //
    // Find Vmax and Vmin
    //
//    if(Va_pu > Vb_pu)
//    {
//        Vmax_pu = Va_pu;
//        Vmin_pu = Vb_pu;
//    }
//    else
//    {
//        Vmax_pu = Vb_pu;
//        Vmin_pu = Va_pu;
//    }
//
//    if(Vc_pu > Vmax_pu)
//    {
//        Vmax_pu = Vc_pu;
//    }
//    else if(Vc_pu < Vmin_pu)
//    {
//        Vmin_pu = Vc_pu;
//    }
//    else
//    {
//        //
//        // Do nothing as if now
//        //
//        ;
//    }

    Vmax_pu = FMAX(FMAX(Va_pu, Vb_pu), Vc_pu);
    Vmin_pu = FMIN(FMIN(Va_pu, Vb_pu), Vc_pu);

    //
    // Compute Vcom = 0.5*(Vmax+Vmin)
    //
    Vcom_pu = (float32_t)0.5 * (Vmax_pu + Vmin_pu);

    //
    // Subtract common-mode term to achieve SV modulation
    //
    pwm->Vabc_pu[0] = (Va_pu - Vcom_pu);
    pwm->Vabc_pu[1] = (Vb_pu - Vcom_pu);
    pwm->Vabc_pu[2] = (Vc_pu - Vcom_pu);

    return;
} // end of SVGEN_run() function

static inline void PWM_clamp(PWMData_t * pwm)
{
    pwm->Vabc_pu[0] = FMAX(FMIN(pwm->Vabc_pu[0], pwm->modulationLimit), -pwm->modulationLimit);
    pwm->Vabc_pu[1] = FMAX(FMIN(pwm->Vabc_pu[1], pwm->modulationLimit), -pwm->modulationLimit);
    pwm->Vabc_pu[2] = FMAX(FMIN(pwm->Vabc_pu[2], pwm->modulationLimit), -pwm->modulationLimit);
}

//*****************************************************************************
//
//! \brief     Runs a first-order filter of the form
//!            y[n] = b0*x[n] + b1*x[n-1] - a1*y[n-1]
//!
//! \param[in] handle      The filter handle
//!
//! \param[in] inputValue  The input value to filter
//!
//! \return    The output value from the filter
//
//*****************************************************************************
static inline float32_t FILTER_FO_run(FILTER_FO_Obj* filter, const float32_t inputValue)
{
    //
    // Compute the output
    //
    filter->y1 = (filter->b0 * inputValue) + (filter->b1 * filter->x1) - (filter->a1 * filter->y1);
    // y0 = (b0 * inputValue) + (a1 * y1);

    //
    // Store values for next time
    //
    filter->x1 = inputValue;

    return(filter->y1);
} // end of FILTER_FO_run() function



static inline void decoupling_run(Motor_t * in)
{
    in->pi_id.ffwdValue = - in->I_dq_A[1] * in->Ls_q * in->omega_e;
    in->pi_iq.ffwdValue  = (in->I_dq_A[0] * in->Ls_d + in->Phi_e) * in->omega_e;
}


//*****************************************************************************
//
//! \brief     Saturates the input value between the minimum and maximum values
//!
//! \param[in] in   The input value
//!
//! \param[in] max  The maximum value allowed
//!
//! \param[in] min  The minimum value allowed
//!
//! \return    The saturated value
//
//*****************************************************************************
static inline float32_t
MATH_sat(const float32_t in, const float32_t max, const float32_t min)
{
    float32_t out = in;

    // if(in < min)
    // {
    //     out = min;
    // }
    // else if(in > max)
    // {
    //     out = max;
    // }
    // else
    // {
    //     //
    //     // Do nothing as of now
    //     //
    //     ;
    // }

    out = FMAX(out, min);
    out = FMIN(out, max);

    return(out);
} // end of MATH_sat() function

//*****************************************************************************
//
//! \brief     Runs the series form of the PI controller
//!
//! \param[in] handle      The PI controller handle
//!
//! \param[in] refValue    The reference value to the controller
//!
//! \param[in] fbackValue  The feedback value to the controller
//!
//! \param[in] ffwdValue   The feedback value to the controller
//!
//! \param[in] pOutValue   The pointer to the controller output value
//!
//! \return    None
//
//*****************************************************************************
static inline float32_t
PI_run_series(PI_Obj * pi, const float32_t fbackValue)
{
    float32_t Error;
    float32_t Up;
    float32_t OutValue;

    Error = pi->refValue - fbackValue;

    //
    // Compute the proportional output
    //
    Up = pi->Kp * Error;

    //
    // Compute the integral output with saturation
    //
    pi->Ui = MATH_sat(pi->Ui + (pi->Ki * Up),pi->outMax,pi->outMin);

    //
    // Saturate the output
    //
    OutValue = MATH_sat(Up + pi->Ui + pi->ffwdValue,pi->outMax,pi->outMin);

    return OutValue;
} // end of PI_run_series() function


extern void FILTER_FO_init(FILTER_FO_Obj* filter, const float32_t bandwidth_Hz, const float32_t sample_freq_Hz);
extern void motor_init(Motor_t * in);


#endif /* FOC_H_ */
