//#############################################################################
//
// FILE:   epwm_ex2_updown_aq.c
//
// TITLE:  ePWM Action Qualifier Module - Using up/down count.
//
//! \addtogroup driver_example_list
//! <h1> ePWM Up Down Count Action Qualifier</h1>
//!
//! This example configures ePWM1, ePWM2, ePWM3 to produce a waveform with
//! independent modulation on ePWMxA and ePWMxB.
//!
//! The compare values CMPA and CMPB are modified within the ePWM's ISR.
//!
//! The TB counter is in up/down count mode for this example.
//!
//! View the ePWM1A/B(GPIO0 & GPIO1), ePWM2A/B(GPIO2 &GPIO3)
//! and ePWM3A/B(GPIO4 & GPIO5) waveforms on oscilloscope.
//
//#############################################################################
// $TI Release: F2837xD Support Library v3.09.00.00 $
// $Release Date: Thu Mar 19 07:35:24 IST 2020 $
// $Copyright:
// Copyright (C) 2013-2020 Texas Instruments Incorporated - http://www.ti.com/
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met:
// 
//   Redistributions of source code must retain the above copyright 
//   notice, this list of conditions and the following disclaimer.
// 
//   Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the 
//   documentation and/or other materials provided with the   
//   distribution.
// 
//   Neither the name of Texas Instruments Incorporated nor the names of
//   its contributors may be used to endorse or promote products derived
//   from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// $
//#############################################################################

//
// Included Files
//
#include "driverlib.h"
#include "device.h"

#include "hvp045a_io.h"
#include "ucc5870.h"
#include "foc.h"
#include "rampgen.h"

//
// This line sets the SAMPLING FREQUENCY to one of the available choices
//
#define  SINGLE_SAMPLING        1
#define  DOUBLE_SAMPLING        2

#define  SAMPLING_METHOD     SINGLE_SAMPLING
//#define  SAMPLING_METHOD     DOUBLE_SAMPLING

#define MOTOR_PWM_DB        60
#define INV_PWM_TICKS       10000
#define INV_PWM_HALF_TBPRD  2500

#define  SET_PIN(PIN,A)  GPIO_writePin(PIN, A)

#define  LED_ON()        SET_PIN(LED_GPIO, 1)
#define  LED_OFF()       SET_PIN(LED_GPIO, 0)

#ifdef _FLASH
#ifndef __cplusplus
#pragma CODE_SECTION(motorControlISR,".TI.ramfunc");
#endif
#endif
__interrupt void motorControlISR(void);

void configureSPI(uint32_t base);
void configureSPI_GPIO(void);
void configureMotorPWM_GPIO(void);
void configureMiscGPIO(void);
void configurePWM(void);
void LED_blink(uint16_t imax);

uint32_t pwmHandle[3] = {EPWM1_BASE,
                         EPWM2_BASE,
                         EPWM3_BASE
};

uint32_t  motorPwmGpio[6] = {
                         PHASE_UH_GPIO,
                         PHASE_UL_GPIO,
                         PHASE_VH_GPIO,
                         PHASE_VL_GPIO,
                         PHASE_WH_GPIO,
                         PHASE_WL_GPIO
                        };

uint32_t  motorPwmGpio_pwm_cfg[6] = {
                                 PHASE_UH_GPIO_PWM_CFG,
                                 PHASE_UL_GPIO_PWM_CFG,
                                 PHASE_VH_GPIO_PWM_CFG,
                                 PHASE_VL_GPIO_PWM_CFG,
                                 PHASE_WH_GPIO_PWM_CFG,
                                 PHASE_WL_GPIO_PWM_CFG
                                };

uint32_t  motorPwmGpio_gpio_cfg[6] = {
                                 PHASE_UH_GPIO_GPIO_CFG,
                                 PHASE_UL_GPIO_GPIO_CFG,
                                 PHASE_VH_GPIO_GPIO_CFG,
                                 PHASE_VL_GPIO_GPIO_CFG,
                                 PHASE_WH_GPIO_GPIO_CFG,
                                 PHASE_WL_GPIO_GPIO_CFG
                                };

/*-----------------------------------------------------------------------------
Define the structure of run time fault flag
-----------------------------------------------------------------------------*/
union {
    uint32_t  all;      // PWM trip status
    struct {
        UCC5870_Status_e UCC5870_status;
                               // bit   description
        uint16_t  nFLT1L : 1;  //  0    low gate driver fault  - pri side
        uint16_t  nFLT2L : 1;  //  1    low gate driver fault  - sec side
        uint16_t  nFLT1H : 1;  //  2    high gate driver fault - pri side
        uint16_t  nFLT2H : 1;  //  3    high gate driver fault - sec side
    } fault;
} tripFlagDMC;

uint16_t scanInverterFaults = 0;

Motor_t     motor1;
PWMData_t   pwm1;

RAMPGEN     rg1;

float32_t VdTesting = 0.0;          // Vd reference (pu)
float32_t VqTesting = 0.5;         // Vq reference (pu)
float32_t speedRef  = 2.0;          // For Closed Loop tests

//
// Main
//
void main(void)
{
    //
    // Initialize device clock and peripherals
    //
    Device_init();

    //
    // Disable pin locks and enable internal pull ups.
    //
    Device_initGPIO();

    //
    // Initialize PIE and clear PIE registers. Disables CPU interrupts.
    //
    Interrupt_initModule();

    //
    // Initialize the PIE vector table with pointers to the shell Interrupt
    // Service Routines (ISR).
    //
    Interrupt_initVectorTable();

    // ****************************************************************************
    // ****************************************************************************
    // Misc GPIO Configuration (Non PWM GPIOs)
    // ****************************************************************************
    // ****************************************************************************
    configureMiscGPIO();

    // ****************************************************************************
    // ****************************************************************************
    // SPI Configuration
    // ****************************************************************************
    // ****************************************************************************
    configureSPI_GPIO();
    configureSPI(GD_SPI_BASE);

    // ****************************************************************************
    // ****************************************************************************
    // UCC5870 Configuration
    // ****************************************************************************
    // ****************************************************************************
    Init_UCC5870_Regs();
    tripFlagDMC.fault.UCC5870_status = Init_UCC5870(motorPwmGpio, motorPwmGpio_gpio_cfg);
    if (tripFlagDMC.fault.UCC5870_status != ALL_GOOD)
    {
        while (1)
        {
            LED_blink(tripFlagDMC.fault.UCC5870_status);
        }
    }

    configureMotorPWM_GPIO();

    //
    // PWM Configuration
    //
    configurePWM();

    // Setup EPWM1 INT event
    Interrupt_register(INT_EPWM1, &motorControlISR); // PWM1 ISR function handle
    EPWM_clearEventTriggerInterruptFlag(EPWM1_BASE); // clear pending INT

    // Setup EPWM1 INT to generate MotorControlISR
#if(SAMPLING_METHOD == SINGLE_SAMPLING)
    // Select INT @ ctr = 0
    EPWM_setInterruptSource(EPWM1_BASE, EPWM_INT_TBCTR_PERIOD);
#elif(SAMPLING_METHOD == DOUBLE_SAMPLING)
    // Select INT @ ctr = 0 or ctr = prd
    EPWM_setInterruptSource(EPWM1_BASE, EPWM_INT_TBCTR_ZERO_OR_PERIOD);
#endif

    EPWM_setInterruptEventCount(EPWM1_BASE, 1); //  1 for INTFRC to work
    EPWM_enableInterrupt(EPWM1_BASE);           // Enable INT from EPWM1

    // Initialize the motor structure
    motor_init(&motor1);
    pwm1.modulationLimit = 1.0;

    // Initialize the RAMPGEN module
    rg1.StepAngleMax = 0.0001;
    rg1.Angle = 0;
    rg1.Out = 0;
    rg1.Gain = 1.0;
    rg1.Offset = 1.0;

    //
    // Enable sync and clock to PWM
    //
    SysCtl_enablePeripheral(SYSCTL_PERIPH_CLK_TBCLKSYNC);

    //
    // Enable ePWM interrupts
    //
    Interrupt_enable(INT_EPWM1);

    //
    // Enable Global Interrupt (INTM) and realtime interrupt (DBGM)
    //
    EINT;
    ERTM;

    //
    // IDLE loop. Just sit and loop forever (optional):
    //
    for(;;)
    {
        if (scanInverterFaults)
        {
            scanInverterFaults = 0;

            //
            // Diagnose from physical fault signals from various sources
            //
            tripFlagDMC.fault.nFLT1H = GPIO_readPin(nFLT1H_GPIO);
            tripFlagDMC.fault.nFLT2H = GPIO_readPin(nFLT2H_GPIO);
            tripFlagDMC.fault.nFLT1L = GPIO_readPin(nFLT1L_GPIO);
            tripFlagDMC.fault.nFLT2L = GPIO_readPin(nFLT2L_GPIO);

            //
            // diagnose inverter - update status_fault, pri_rdy_Fault, sec_rdy_fault
            //
            tripFlagDMC.fault.UCC5870_status = inverterDiagnostics();
        }
    }
}

//
// epwm1ISR - ePWM 1 ISR
//
__interrupt void motorControlISR(void)
{
    // ----------------------------------------------------------------------------
    //  Connect inputs of the RAMP GEN module and call the ramp generator module
    // ----------------------------------------------------------------------------
    rg1.Freq = speedRef;
    fclRampGen(&rg1);

    motor1.theta_e  = rg1.Out;

    motor1.Sine = __sinpuf32(motor1.theta_e);
    motor1.Cosine = __cospuf32(motor1.theta_e);

    motor1.Vout_dq_V[0] = VdTesting;
    motor1.Vout_dq_V[1] = VqTesting;

    motor1.oneOverDcBus_invV = 1.0;

    // Generate SVPWM
    ipark_run(&motor1);
    SVGEN_run(&motor1, &pwm1);

    PWM_clamp(&pwm1);

    EPWM_setCounterCompareValue(EPWM1_BASE, EPWM_COUNTER_COMPARE_A,
                        (uint16_t)((INV_PWM_HALF_TBPRD * pwm1.Vabc_pu[0]) +
                                    INV_PWM_HALF_TBPRD));
    EPWM_setCounterCompareValue(EPWM2_BASE, EPWM_COUNTER_COMPARE_A,
                        (uint16_t)((INV_PWM_HALF_TBPRD * pwm1.Vabc_pu[1]) +
                                    INV_PWM_HALF_TBPRD));
    EPWM_setCounterCompareValue(EPWM3_BASE, EPWM_COUNTER_COMPARE_A,
                        (uint16_t)((INV_PWM_HALF_TBPRD * pwm1.Vabc_pu[2]) +
                                    INV_PWM_HALF_TBPRD));

    //
    // Clear INT flag for this timer
    //
    EPWM_clearEventTriggerInterruptFlag(EPWM1_BASE);

    //
    // Acknowledge interrupt group
    //
    Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP3);
}


#define  SPIFSI_CLK_SPEED    4000000U

void configureSPI(uint32_t base)
{
    // Must put SPI into reset before configuring it
    SPI_disableModule(base);

    // SPI configuration.
    SPI_setConfig(base, DEVICE_LSPCLK_FREQ, SPI_PROT_POL0PHA0,
                  SPI_MODE_MASTER, SPIFSI_CLK_SPEED, 16);
    SPI_disableLoopback(base);
    SPI_setEmulationMode(base, SPI_EMULATION_STOP_MIDWAY);
                               //SPI_EMULATION_FREE_RUN);

    // FIFO disable
    SPI_disableFIFO(base);

    // Interrupt configuration
    SPI_disableInterrupt(base, SPI_INT_RX_DATA_TX_EMPTY);
    SPI_disableInterrupt(base, SPI_INT_RX_OVERRUN);

    // Configuration complete. Enable the module.
    SPI_enableModule(base);
}

void configureSPI_GPIO(void)
{
    // SPIA-SIMO
    GPIO_setMasterCore(GD_SPISIMO_GPIO, GPIO_CORE_CPU1);
    GPIO_setPinConfig(GD_SPISIMO_GPIO_CFG);

    // SPIA-SOMI
    GPIO_setMasterCore(GD_SPISOMI_GPIO, GPIO_CORE_CPU1);
    GPIO_setQualificationMode(GD_SPISOMI_GPIO, GPIO_QUAL_3SAMPLE);
    GPIO_setPinConfig(GD_SPISOMI_GPIO_CFG);

    // SPIA-CLK
    GPIO_setMasterCore(GD_SPICLK_GPIO, GPIO_CORE_CPU1);
    GPIO_setPinConfig(GD_SPICLK_GPIO_CFG);

    // SPIA-STE
    GPIO_setMasterCore(GD_SPISTE_GPIO, GPIO_CORE_CPU1);
    GPIO_setPinConfig(GD_SPISTE_GPIO_CFG);

    return;
}

void configureMotorPWM_GPIO(void)
{
    uint16_t  i;

    //
    // Motor inverter PWM GPIO init
    //
    for (i=0; i<6; i++)
    {
        GPIO_setMasterCore(motorPwmGpio[i], GPIO_CORE_CPU1);
        GPIO_setPadConfig (motorPwmGpio[i], GPIO_PIN_TYPE_STD);
        GPIO_setPinConfig (motorPwmGpio_pwm_cfg[i]);
    }

    return;
}

void configureMiscGPIO(void)
{
    // GPIO31->LED
    GPIO_setMasterCore(LED_GPIO, GPIO_CORE_CPU1);
    GPIO_setPadConfig(LED_GPIO, GPIO_PIN_TYPE_STD);
    GPIO_setPinConfig(LED_GPIO_CFG);
    GPIO_setDirectionMode(LED_GPIO, GPIO_DIR_MODE_OUT);


    // Configure GPIO used for Trip Mechanism with UCC5870

//    // GPIO86 - ASC EN
//    GPIO_setMasterCore(ASC_EN_GPIO, GPIO_CORE_CPU1);
//    GPIO_setPadConfig(ASC_EN_GPIO, GPIO_PIN_TYPE_STD);
//    GPIO_setDirectionMode(ASC_EN_GPIO, GPIO_DIR_MODE_OUT);
//    GPIO_setPinConfig(ASC_EN_GPIO_CFG);
//
//    // GPIO88 - ASC L
//    GPIO_setMasterCore(ASC_L_GPIO, GPIO_CORE_CPU1);
//    GPIO_setPadConfig(ASC_L_GPIO, GPIO_PIN_TYPE_STD);
//    GPIO_setDirectionMode(ASC_L_GPIO, GPIO_DIR_MODE_OUT);
//    GPIO_setPinConfig(ASC_L_GPIO_CFG);
//
//    // GPIO90 - ASC H
//    GPIO_setMasterCore(ASC_H_GPIO, GPIO_CORE_CPU1);
//    GPIO_setPadConfig(ASC_H_GPIO, GPIO_PIN_TYPE_STD);
//    GPIO_setDirectionMode(ASC_H_GPIO, GPIO_DIR_MODE_OUT);
//    GPIO_setPinConfig(ASC_H_GPIO_CFG);

    //
    // Fault signals from UCC5870 will connect to TRIP thru inputXbar
    //

    // nFLT2 L  (1/0 --> GOOD/FAULT)
    GPIO_setMasterCore(nFLT2L_GPIO, GPIO_CORE_CPU1);
    GPIO_setPadConfig(nFLT2L_GPIO, GPIO_PIN_TYPE_INVERT);
    GPIO_setDirectionMode(nFLT2L_GPIO, GPIO_DIR_MODE_IN);
    GPIO_setQualificationMode(nFLT2L_GPIO, GPIO_QUAL_3SAMPLE);
    GPIO_setPinConfig(nFLT2L_GPIO_CFG);

    // nFLT1 L  (1/0 --> GOOD/FAULT)
    GPIO_setMasterCore(nFLT1L_GPIO, GPIO_CORE_CPU1);
    GPIO_setPadConfig(nFLT1L_GPIO, GPIO_PIN_TYPE_INVERT);
    GPIO_setDirectionMode(nFLT1L_GPIO, GPIO_DIR_MODE_IN);
    GPIO_setQualificationMode(nFLT1L_GPIO, GPIO_QUAL_3SAMPLE);
    GPIO_setPinConfig(nFLT1L_GPIO_CFG);

    // nFLT2 H  (1/0 --> GOOD/FAULT)
    GPIO_setMasterCore(nFLT2H_GPIO, GPIO_CORE_CPU1);
    GPIO_setPadConfig(nFLT2H_GPIO, GPIO_PIN_TYPE_INVERT);
    GPIO_setDirectionMode(nFLT2H_GPIO, GPIO_DIR_MODE_IN);
    GPIO_setQualificationMode(nFLT2H_GPIO, GPIO_QUAL_3SAMPLE);
    GPIO_setPinConfig(nFLT2H_GPIO_CFG);

    // nFLT1 H  (1/0 --> GOOD/FAULT)
    GPIO_setMasterCore(nFLT1H_GPIO, GPIO_CORE_CPU1);
    GPIO_setPadConfig(nFLT1H_GPIO, GPIO_PIN_TYPE_INVERT);
    GPIO_setDirectionMode(nFLT1H_GPIO, GPIO_DIR_MODE_IN);
    GPIO_setQualificationMode(nFLT1H_GPIO, GPIO_QUAL_3SAMPLE);
    GPIO_setPinConfig(nFLT1H_GPIO_CFG);

    return;
}

//
// PWM Configuration
//
void configurePWM(void)
{
    uint16_t base;

    SysCtl_disablePeripheral(SYSCTL_PERIPH_CLK_TBCLKSYNC);

    // *****************************************
    // Inverter PWM configuration - PWM 1, 2, 3
    // *****************************************
    for(base = 0; base < 3; base++)
    {
        // Time Base SubModule Registers
        // set Immediate load
//        EPWM_setPeriodLoadMode(pwmHandle[base], EPWM_PERIOD_DIRECT_LOAD);
        EPWM_setPeriodLoadMode(pwmHandle[base], EPWM_PERIOD_SHADOW_LOAD);
        EPWM_setTimeBasePeriod(pwmHandle[base], INV_PWM_TICKS / 2);
        EPWM_setPhaseShift(pwmHandle[base], 0);
        EPWM_setTimeBaseCounter(pwmHandle[base], 0);
        EPWM_setTimeBaseCounterMode(pwmHandle[base], EPWM_COUNTER_MODE_UP_DOWN);
        EPWM_setClockPrescaler(pwmHandle[base], EPWM_CLOCK_DIVIDER_1,
                               EPWM_HSCLOCK_DIVIDER_1);

        EPWM_disablePhaseShiftLoad(pwmHandle[base]);

        // sync "down-stream"
        EPWM_setSyncOutPulseMode(pwmHandle[base],
                                      EPWM_SYNC_OUT_PULSE_ON_COUNTER_ZERO);

        // Counter Compare Submodule Registers
        // set duty 0% initially
        EPWM_setCounterCompareValue(pwmHandle[base], EPWM_COUNTER_COMPARE_A, 0);

#if(SAMPLING_METHOD == SINGLE_SAMPLING)
        EPWM_setCounterCompareShadowLoadMode(pwmHandle[base],
                                             EPWM_COUNTER_COMPARE_A,
                                             EPWM_COMP_LOAD_ON_CNTR_ZERO);
#elif(SAMPLING_METHOD == DOUBLE_SAMPLING)
        EPWM_setCounterCompareShadowLoadMode(pwmHandle[base],
                                             EPWM_COUNTER_COMPARE_A,
                                             EPWM_COMP_LOAD_ON_CNTR_ZERO_PERIOD);
#endif

        // Action Qualifier SubModule Registers
        EPWM_setActionQualifierActionComplete(pwmHandle[base], EPWM_AQ_OUTPUT_A,
                (EPWM_ActionQualifierEventAction)(EPWM_AQ_OUTPUT_LOW_UP_CMPA |
                                               EPWM_AQ_OUTPUT_HIGH_DOWN_CMPA));

        // Active high complementary PWMs - Set up the deadband
        EPWM_setRisingEdgeDeadBandDelayInput(pwmHandle[base],
                                             EPWM_DB_INPUT_EPWMA);
        EPWM_setFallingEdgeDeadBandDelayInput(pwmHandle[base],
                                              EPWM_DB_INPUT_EPWMA);

        EPWM_setDeadBandDelayMode(pwmHandle[base], EPWM_DB_RED, true);
        EPWM_setDeadBandDelayMode(pwmHandle[base], EPWM_DB_FED, true);
        EPWM_setDeadBandDelayPolarity(pwmHandle[base], EPWM_DB_RED,
                                      EPWM_DB_POLARITY_ACTIVE_HIGH);
        EPWM_setDeadBandDelayPolarity(pwmHandle[base],
                                      EPWM_DB_FED, EPWM_DB_POLARITY_ACTIVE_LOW);
        EPWM_setRisingEdgeDelayCount(pwmHandle[base], MOTOR_PWM_DB);
        EPWM_setFallingEdgeDelayCount(pwmHandle[base], MOTOR_PWM_DB);
    }

    // configure 2 and 3 as slaves
    EPWM_enablePhaseShiftLoad(EPWM2_BASE);
    EPWM_setPhaseShift(EPWM2_BASE, 2);
    EPWM_setCountModeAfterSync(EPWM2_BASE, EPWM_COUNT_MODE_UP_AFTER_SYNC);

    EPWM_enablePhaseShiftLoad(EPWM3_BASE);
    EPWM_setPhaseShift(EPWM3_BASE, 2);
    EPWM_setCountModeAfterSync(EPWM3_BASE, EPWM_COUNT_MODE_UP_AFTER_SYNC);

    SysCtl_disablePeripheral(SYSCTL_PERIPH_CLK_TBCLKSYNC);

    return;
}

void LED_blink(uint16_t imax)
{
    uint16_t i;

    for (i=0; i<imax; i++)
    {
        LED_ON();
        DEVICE_DELAY_US(100000);
        LED_OFF();
        DEVICE_DELAY_US(100000);
    }
    DEVICE_DELAY_US(10000);

    return;
}

